// Window
var window = {};

// Module
var module = { exports: { } };

// Exports
var exports = module.exports;

// Console: test only
var console = { log: __logNative__ };

// Require
function require(path, env) {

  env = env || "";
  path = __resolvePath__(path, env);

  if (require.cache[path]) {
    return require.cache[path].exports;
  }

  var source = __includeCode__(path, env);
  var module = { exports: { } };
  
  var execute = new Function("exports, module", source);
  execute(module.exports, module);
  
  require.cache[path] = module;
  return module.exports;
}

// Methods

function $type(props, inherited, defs) {
  var results = {};

  for (var key in inherited) {
    results[key] = inherited[key];
  }

  for (var key in props) {
    results[key] = props[key];
  }

  for (var key in defs) {
    results[key] = defs[key];
  }

  return results;
}

function $class(static, proto, defs) {
  var results = {};

  for (var key in static) {
    results[key] = static[key];
  }

  if (proto) {
    results["prototype"] = {};
  }

  for (var key in proto) {
    results["prototype"][key] = proto[key];
  }

  for (var key in defs) {
    results[key] = defs[key];
  }

  return results;
}

function $(type, doc, url) {

  if (url && url.lastIndexOf("http", 0) !== 0) {
    url = "https://docs.xteko.com/#/en" + url;
  }

  return { "!type": type, "!doc": doc, "!url": url };
}

function $dict(type, values) {
  var types = require("./types/types");
  types[type] = $type(values);
  return type; 
}

function $promise(type, values) {
  $dict(type, values);
  return " -> +Promise[:t=" + type + "]";
}

// Cache
require.cache = Object.create(null);

// Global tern
var tern = require("./lib/tern");

// Require
require("./plugin/commonjs");
require("./plugin/doc_comment");
require("./plugin/complete_strings");
require("./types/loader");

var config = {
  getFile: __getFile__,
  defs: ["ecmascript"].map(function(name) {
    return __getDefs__(name);
  }),
  plugins: {
    "commonjs": true,
    "doc_comment": true,
    "complete_strings": true
  }
};

// Tern service
var service = new tern.Server(config);
var walker = require("./estraverse");

// Collect function nodes
var funcDeclarations = {};
service.on("postParse", function(ast) {
  var source = ast.sourceFile.name;
  funcDeclarations[source] = [];
  walker.traverse(ast, {
    enter: function(node) {
      if (node.type === "FunctionDeclaration") {
        funcDeclarations[source].push({
          "name": node.id.name,
          "params": node.params.map(function(item) {
            return item.name;
          }),
          "start": node.id.start,
          "end": node.id.end,
          "source": {
            "name": node.id.sourceFile.name,
            "text": node.id.sourceFile.text,
          },
          "async": node.async,
        });
      }
    }
  });
});

// JSBox defs
var defs = require("./types/defs");
service.addDefs(defs);

// JavaScriptCore APIs

function intellisense(anchor, roughly, callback) {

  var options = {
    query: {
      type: "completions",
      filter: true,
      types: true,
      docs: true,
      urls: true,
      includeKeywords: true,
      guess: !roughly,
      caseInsensitive: true,
      file: anchor.file,
      end: anchor.end
    },
    files: [
      {
        type: "full",
        name: anchor.file,
        text: anchor.text
      }
    ]
  };
  
  service.request(options, callback);
}

function typeInfer(anchor, callback) {

  var options = {
    query: {
      type: "type",
      preferFunction: true,
      file: anchor.file,
      start: anchor.start,
      end: anchor.end
    },
    files: [
      {
        type: "full",
        name: anchor.file,
        text: anchor.text
      }
    ]
  };
  
  service.request(options, callback);
}

function identityInfer(anchor, type, callback) {

  var options = {
    query: {
      type: ["definition", "documentation", "refs"][type],
      file: anchor.file,
      start: anchor.start,
      end: anchor.end
    }
  };

  service.request(options, callback);
}

function renameInfer(anchor, newName, callback) {
  
  var options = {
    query: {
      type: "rename",
      file: anchor.file,
      start: anchor.start,
      end: anchor.end,
      newName: newName
    }
  };

  service.request(options, callback);
}

function propertiesInfer(prefix, callback) {

  var options = {
    query: {
      type: "properties",
      prefix: prefix
    }
  };

  service.request(options, callback);
}

function filesInfer(callback) {
  
  var options = {
    query: {
      type: "files"
    }
  };

  service.request(options, callback);
}

function clearFunctionDeclarations() {
  funcDeclarations = {};
}

// Module

function registerModule(name, text, parent) {
  service.delFile(name);
  service.addFile(name, text, parent);
}

function unregisterModules() {
  filesInfer(function(error, results) {
    if (results && results.files) {
      results.files.forEach(function(name) {
        service.delFile(name);
      });
    }
  });
}

// Browser

function setWebAPIsEnabled(enabled) {
  var name = "browser";
  if (enabled) {
    service.addDefs(__getDefs__(name));
  } else {
    service.deleteDefs(name);
  }
}
