var types = require("./types");
var defs = require("./defs");

// $(id)
defs["$"] = $("fn(id: string) -> $view", "Get a view with id.", "/uikit/render?id=id");

// $include
defs["$include"] = $("fn(path: string)", "Include a module in current JavaScript context.", "/package/module?id=include39path39");

// $l10n
defs["$l10n"] = $("fn(key: string) -> string", "Get localized string with key.", "/foundation/l10n?id=l10n");

// $delay
defs["$delay"] = $("fn(sec: number, block: fn()) -> $cancelable", "Run task after delay easily.", "/foundation/thread?id=delaynumber-function");

// $wait
defs["$wait"] = $("fn(sec: number) -> +Promise", "Wait for a delay.", "/foundation/thread?id=waitsec");

// $rect
defs["$rect"] = $("fn(x: number, y: number, width: number, height: number) -> $rect", "Create a rectangle.", "/function/index?id=rectx-y-width-height");

// $size
defs["$size"] = $("fn(width: number, height: number) -> $size", "Create a size.", "/function/index?id=sizewidth-height");

// $point
defs["$point"] = $("fn(x: number, y: number) -> $point", "Creat a point.", "/function/index?id=pointx-y");

// $insets
defs["$insets"] = $("fn(top: number, left: number, bottom: number, right: number) -> $insets", "Create an edge insets.", "/function/index?id=insetstop-left-bottom-right");

// $color
defs["$color"] = $("fn(hex: string) -> $color", "Create a color with hex string or name.", "/function/index?id=colorstring");

// $rgb
defs["$rgb"] = $("fn(r: number, g: number, b: number) -> $color", "Create a color with red, green, blue values.", "/function/index?id=rgbred-green-blue");

// $rgba
defs["$rgba"] = $("fn(r: number, g: number, b: number, a: number) -> $color", "Create a color with red, green, blue and alpha channel.", "/function/index?id=rgbared-green-blue-alpha");

// $font
defs["$font"] = $("fn(name: string, size?: number) -> $font", "Create a font, name is an optional parameter.", "/function/index?id=fontname-size");

// $range
defs["$range"] = $("fn(loc: number, len: number) -> $range", "Create a range.", "/function/index?id=rangelocation-length");

// $indexPath
defs["$indexPath"] = $("fn(section: number, row: number) -> $indexPath", "Create an indexPath.", "/function/index?id=indexpathsection-row");

// $data
defs["$data"] = $("fn(args: ?) -> $data", "Create a binary data.", "/function/index?id=dataobject");

// $icon
defs["$icon"] = $("fn(id: string, color?: $color, size?: $size) -> $icon", "Get an icon provided by JSBox.", "/function/index?id=iconcode-color-size");

// $objc_retain
defs["$objc_retain"] = $("fn(obj: object)", "Retain an Objective-C Runtime object.", "/function/index?id=objc_retainobject");

// $objc_release
defs["$objc_release"] = $("fn(obj: object)", "Release a Runtime object manually.", "/function/index?id=objc_relaseobject");

// $get_protocol
defs["$get_protocol"] = $("fn(name: string) -> ?", "Get an Objective-C Protocol.", "/function/index?id=get_protocolname");

// $objc_clean
defs["$objc_clean"] = $("fn()", "Clean all Objective-C definitions.", "/function/index?id=objc_clean");

// $props
defs["$props"] = $("fn(obj: object) -> object", "$props helps you get all properties of an object.", "/data/object?id=props");
defs["$desc"] = $("fn(obj: object) -> string", "$desc returns description of an object.", "/data/object?id=desc");

// $defc
defs["$defc"] = $("fn(name: string, types: string)", "Include a C function.", "/runtime/c?id=defc");

// $block
defs["$block"] = $("fn(types: string, block: fn(args: ?)) -> object", "Declare an Objective-C block", "/runtime/blocks?id=block");

// $env
defs["$env"] = $class({
  "app": "number",
  "today": "number",
  "action": "number",
  "safari": "number",
  "notification": "number",
  "keyboard": "number",
  "siri": "number",
  "all": "number",
}, null, $(null, "JSBox environment.", "/data/constant?id=env"));

// $align
defs["$align"] = $class({
  "left": "number",
  "center": "number",
  "right": "number",
  "justified": "number",
  "natural": "number",
}, null, $(null, "iOS text alignment type.", "/data/constant?id=align"));

// $contentMode
defs["$contentMode"] = $class({
  "scaleToFill": "number",
  "scaleAspectFit": "number",
  "scaleAspectFill": "number",
  "redraw": "number",
  "center": "number",
  "top": "number",
  "bottom": "number",
  "left": "number",
  "right": "number",
}, null, $(null, "iOS view content mode.", "/data/constant?id=contentmode"));

// $btnType
defs["$btnType"] = $class({
  "custom": "number",
  "system": "number",
  "disclosure": "number",
  "infoLight": "number",
  "infoDark": "number",
  "contactAdd": "number",
}, null, $(null, "iOS button type.", "/data/constant?id=btntype"));

// $alertActionType
defs["$alertActionType"] = $class({
  "default": "number",
  "cancel": "number",
  "destructive": "number",
}, null, $(null, "Alert action type.", "/data/constant?id=alertactiontype"));

// $zero
defs["$zero"] = $class({
  "point": "$point",
  "size": "$size",
  "rect": "$rect",
  "insets": "$insets",
}, null, $(null, "Zero values.", "/data/constant?id=zero"));

// $layout
defs["$layout"] = $class({
  "fill": "$constraint",
  "center": "$constraint",
}, null, $(null, "Common layout functions.", "/data/constant?id=layout"));

// $lineCap
defs["$lineCap"] = $class({
  "butt": "number",
  "round": "number",
  "square": "number",
}, null, $(null, "iOS line cap.", "/data/constant?id=linecap"));

// $lineCap
defs["$lineCap"] = $class({
  "miter": "number",
  "round": "number",
  "bevel": "number",
}, null, $(null, "iOS line join.", "/data/constant?id=linejoin"));

// $mediaType
defs["$mediaType"] = $class({
  "image": "string",
  "jpeg": "string",
  "jpeg2000": "string",
  "tiff": "string",
  "pict": "string",
  "gif": "string",
  "png": "string",
  "icns": "string",
  "bmp": "string",
  "ico": "string",
  "raw": "string",
  "live": "string",
  "movie": "string",
  "video": "string",
  "audio": "string",
  "mov": "string",
  "mpeg": "string",
  "mpeg2": "string",
  "mp3": "string",
  "mp4": "string",
  "avi": "string",
  "wav": "string",
  "midi": "string",
}, null, $(null, "UTI Types.", "/data/constant?id=mediatype"));

// $imgPicker
defs["$imgPicker"] = $class({
  "quality": "object",
  "captureMode": "object",
  "device": "object",
  "flashMode": "object",
}, null, $(null, "iOS image picker constants.", "/data/constant?id=imgpicker"));

// $kbType
defs["$kbType"] = $class({
  "default": "number",
  "ascii": "number",
  "nap": "number",
  "url": "number",
  "number": "number",
  "phone": "number",
  "namePhone": "number",
  "email": "number",
  "decimal": "number",
  "twitter": "number",
  "search": "number",
  "asciiPhone": "number",
}, null, $(null, "iOS keyboard types.", "/data/constant?id=kbtype"));

// $assetMedia
defs["$assetMedia"] = $class({
  "type": "object",
  "subType": "object",
}, null, $(null, "iOS asset media constants.", "/data/constant?id=assetmedia"));

// $pageSize
defs["$pageSize"] = $class({
  "letter": "number", "governmentLetter": "number", "legal": "number", "juniorLegal": "number", "ledger": "number", "tabloid": "number",
  "A0": "number", "A1": "number", "A2": "number", "A3": "number", "A4": "number", "A5": "number", "A6": "number", "A7": "number", "A8": "number", "A9": "number", "A10": "number",
  "B0": "number", "B1": "number", "B2": "number", "B3": "number", "B4": "number", "B5": "number", "B6": "number", "B7": "number", "B8": "number", "B9": "number", "B10": "number",
  "C0": "number", "C1": "number", "C2": "number", "C3": "number", "C4": "number", "C5": "number", "C6": "number", "C7": "number", "C8": "number", "C9": "number", "C10": "number",
  "custom": "number",
}, null, $(null, "$pdf page sizes.", "/data/constant?id=pagesize"));
