var types = require("./types");
var defs = require("./defs");

// object
types["object"] = $type({}, null, $(null, "Generic JavaScript object."));

// Error
types["$error"] = $type({
  "domain": "string",
  "code": "number",
  "userInfo": "object",
  "localizedDescription": "string",
  "localizedFailureReason": "string",
  "localizedRecoverySuggestion": "string",
  "localizedRecoveryOptions": "[string]",
  "recoveryAttempter": "object",
  "helpAnchor": "string",
}, null, $(null, "An object that contains error code and message.", "https://docs.xteko.com/#/en/object/error"));

// URL
types["$url"] = $type({
  "string": "string",
  "scheme": "string",
  "resourceSpecifier": "string",
  "host": "string",
  "port": "object",
  "user": "string",
  "password": "string",
  "path": "path",
  "fragment": "string",
  "parameterString": "string",
  "query": "string",
  "relativePath": "string",
}, null, $(null, "An object that indicates an URL."));

// HTTPURLResponse
types["$httpResponse"] = $type({
  "url": "string",
  "MIMEType": "string",
  "expectedContentLength": "number",
  "textEncodingName": "string",
  "suggestedFilename": "string",
  "statusCode": "number",
  "headers": "object",
}, null, $(null, "Response object that is returned from a HTTP request.", "/object/response"));

// IndexPath
types["$indexPath"] = $type({
  "section": "number",
  "row": "number",
  "item": "number",
}, null, $(null, "Indicates a position (section, row) in list view or matrix view.", "/object/index-path"));

// Data
types["$data"] = $type({
  "info": "object",
  "string": "string",
  "image": "$image",
  "fileName": "string",
  "append": $("fn(data: $data) -> $data", "Append another data, returns a new data."),
  "subdata": $("fn(range: $range) -> $data", "Get subdata with a given range."),
  "toString": $("fn(encoding: number) -> string", "Convert to string with encoding."),
}, null, $(null, "Represents a binary data (file).", "/object/data"));

// Image
types["$image"] = $type({
  "size": "$size",
  "orientation": "number",
  "orientationFixedImage": "$image",
  "info": "object",
  "scale": "number",
  "png": "$data",
  "averageColor": "$color",
  "resized": $("fn(size: $size) -> $image", "Get a resized image."),
  "jpg": $("fn(quality: number) -> $data", "Convert to JPEG data with quality."),
  "colorAtPixel": $("fn(pt: $point) -> $color", "Get the color at pixel."),
}, null, $(null, "Image object that can be displayed in views.", "/object/image"));

// Color
types["$color"] = $type({
  "hexCode": "string",
  "components": "object",
}, null, $(null, "Color object that can be used as background etc.", "/object/color"));

// Font
types["$font"] = $type(null, null, $(null, "Font for text rendering.", "/data/method?id=fontname-size"));

// Icon
types["$icon"] = $type(null, null, $(null, "JSBox built-in icon.", "/data/method?id=iconcode-color-size"));

// Timer
types["$timer"] = $type({
  "fire": $("fn()", "Fire the event."),
  "invalidate": $("fn()", "Invalidate the timer."),
}, null, $(null, "Timer object that holds a scheduled task."));

// Addin
types["$addin"] = $type({
  "name": "string",
  "url": "string",
  "data": "$data",
  "id": "string",
  "version": "string",
  "icon": "string",
  "summary": "string",
  "author": "string",
  "website": "string",
  "types": "number",
}, null, $(null, "Represents an addin object in JSBox.", "/addin/method"));

// Cancelable Block
types["$cancelable"] = $type({
  "cancel": $("fn()", "Cancel the task")
}, null, $(null, "A cancelable task."));

// $device
defs["$device"] = $class({
  "info": $("object", "Returns the basic info of device.", "/foundation/device?id=deviceinfo"),
  "isIphoneX": $("bool", "Check whether device an iPhone X.", "/foundation/device?id=deviceisxxx"),
  "isIphonePlus": $("bool", "Check whether the screen scale is 3x.", "/foundation/device?id=deviceisxxx"),
  "isIpad": $("bool", "Check whether the device is an iPad.", "/foundation/device?id=deviceisxxx"),
  "isIpadPro": $("bool", "Check whether the device is an iPad Pro.", "/foundation/device?id=deviceisxxx"),
  "space": $("object", "Returns memory usage and disk usage of the device.", "/foundation/device?id=devicespace"),
  "ssid": $("object", "Get the SSID of current Wi-Fi:.", "/foundation/device?id=devicessid"),
  "networkType": $("object", "Returns the network type.", "/foundation/device?id=devicenetworktype"),
  "wlanAddress": $("string", "Get WLAN address.", "/foundation/device?id=devicewlanaddress"),
  "taptic": $("fn(level: number)", "Generate a Taptic Engine Feedback.", "/foundation/device?id=devicetapticnumber"),
}, null, $(null, "Retrieve some useful information of the device, such as language, device model etc.", "/foundation/device"));

// $system
defs["$system"] = $class({
  "brightness": $("number", "Get/Set the brightness of screen.", "/foundation/system?id=systembrightness"),
  "volume": $("number", "Get/Set the volume of speaker (0.0 ~ 1.0).", "/foundation/system?id=systemvolume"),
  "call": $("fn(number: string)", "Make a phone call, similar to $app.openURL('tel:number').", "/foundation/system?id=systemcallnumber"),
  "sms": $("fn(number: string)", "Send a text message, similar to $app.openURL('sms:number').", "/foundation/system?id=systemsmsnumber"),
  "mailto": $("fn(email: string)", "Send an email, similar to $app.openURL('mailto:email').", "/foundation/system?id=systemmailtoemail"),
  "facetime": $("fn(number: string)", "Create a FaceTime session, similar to $app.openURL('facetime:number').", "/foundation/system?id=systemfacetimenumber"),
  "makeIcon": $("fn(args: ?)", "Create home screen icon for url.", "/foundation/system?id=systemmakeiconobject"),
}, null, $(null, "Operating System related APIs", "/foundation/system"));

// $app
defs["$app"] = $class({
  "env": $("number", "Get current environment.", "/foundation/app?id=appenv"),
  "widgetIndex": $("number", "Returns current widget index of JSBox.", "/foundation/app?id=appwidgetindex"),
  "info": $("Object", "Returns the info of JSBox itself.", "/foundation/app?id=appinfo"),
  "idleTimerDisabled": $("bool", "Whether disable auto dimming of the screen.", "/foundation/app?id=appidletimerdisabled"),
  "autoKeyboardEnabled": $("bool", "Manage scroll views automatically, to avoid the keyboard hides text fields.", "/foundation/app?id=appautokeyboardenabled"),
  "keyboardToolbarEnabled": $("bool", "Display a toolbar at the top of keyboard.", "/foundation/app?id=appkeyboardtoolbarenabled"),
  "rotateDisabled": $("bool", "Set to true to disable screen rotating.", "/foundation/app?id=approtatedisabled"),
  "strings": $("object", "Localization strings.", "/foundation/l10n?id=appstrings"),
  "minSDKVer": $("string", "Minimal available version of JSBox.", "/foundation/app?id=appminsdkver"),
  "minOSVer": $("string", "Minimal available version of iOS.", "/foundation/app?id=appminosver"),
  "close": $("fn(sec: number)", "Close current running script after delay.", "/foundation/app?id=appclosedelay"),
  "openURL": $("fn(url: string)", "Open a URL or a URL Scheme.", "/foundation/app?id=appopenurlstring"),
  "openBrowser": $("fn(args: ?)", "Open an URL with external browsers.", "/foundation/app?id=appopenbrowserobject"),
  "listen": $("fn(args: ?)", "Observe notifications posted by JSBox addins.", "/foundation/app?id=applistenobject"),
  "notify": $("fn(args: ?)", "Post a custom event.", "/foundation/app?id=appnotifyobject"),
  "tips": $("fn(tips: string)", "Give user a nice hint.", "/foundation/app?id=apptipsstring"),
}, null, $(null, "Offers APIs that relate to the application and addin itself.", "/foundation/app"));

// $widget
defs["$widget"] = $class({
  "mode": $("number", "Get current display mode (show less/show more).", "/widget/method?id=widgetmode"),
  "modeChanged": $("fn(mode: number)", "Observe mode changes.", "/widget/method?id=widgetmodechanged"),
  "height": $("number", "Get or set the height of widget.", "/widget/method?id=widgetheight"),
}, null, $(null, "Provides widget related functionalities.", "/widget/method"));

// $addin
defs["$addin"] = $class({
  "list": $("[$addin]", "Get all installed addins (scripts).", "/addin/method?id=addinlist"),
  "current": $("$addin", "Get current running addin.", "/addin/method?id=addincurrent"),
  "save": $("fn(args: ?)", "Install a new addin.", "/addin/method?id=addinsaveobject"),
  "delete": $("fn(name: string)", "Delete an installed addin.", "/addin/method?id=addindeletename"),
  "run": $("fn(name: string)", "Run a script with name.", "/addin/method?id=addinrunname"),
  "restart": $("fn()", "Restart current running script.", "/addin/method?id=addinrestart"),
  "compile": $("fn(script: string) -> string", "Convert scripts to JSBox syntax.", "/addin/method?id=addincompilescript"),
  "eval": $("fn(script: string) -> object", "Similar to eval(), but convert script to JSBox syntax first.", "/addin/method?id=addinevalscript"),
}, null, $(null, "We can manage our scripts with $addin APIs.", "/addin/method"));

// $thread
defs["$thread"] = $class({
  "background": $("fn(args: ?) -> +Promise[:t=?]", "Run task on background thread.", "/foundation/thread?id=threadbackgroundobject"),
  "main": $("fn(args: ?) -> +Promise[:t=?]", "Run task on main thread.", "/foundation/thread?id=threadmainobject"),
}, null, $(null, "Thread switching, perform delayed and schedule task.", "/foundation/thread"));

// $clipboard
defs["$clipboard"] = $class({
  "text": $("string", "Get/Set clipboard text.", "/foundation/clipboard?id=clipboardtext"),
  "image": $("$image", "Get/Set clipboard image.", "/foundation/clipboard?id=clipboardimage"),
  "items": $("[?]", "Get/Set clipboard items.", "/foundation/clipboard?id=clipboarditems"),
  "phoneNumbers": $("[string]", "Get all phone numbers from clipboard.", "/foundation/clipboard?id=clipboardphonenumbers"),
  "phoneNumber": $("string", "Get the first phone number from clipboard.", "/foundation/clipboard?id=clipboardphonenumber"),
  "links": $("[string]", "Get all links from clipboard.", "/foundation/clipboard?id=clipboardlinks"),
  "link": $("string", "Get the first link from clipboard.", "/foundation/clipboard?id=clipboardlink"),
  "emails": $("[string]", "Get all emails from clipboard.", "/foundation/clipboard?id=clipboardemails"),
  "email": $("string", "Get the first email from clipboard.", "/foundation/clipboard?id=clipboardemail"),
  "dates": $("[+Date]", "Get all dates from clipboard.", "/foundation/clipboard?id=clipboarddates"),
  "date": $("+Date", "Get the first date from clipboard.", "/foundation/clipboard?id=clipboarddate"),
  "setTextLocalOnly": $("fn(text: string)", "Set text to clipboard, but ignore Universal Clipboard.", "/foundation/clipboard?id=clipboardsettextlocalonlystring"),
  "set": $("fn(args: ?)", "Set clipboard by type and value.", "/foundation/clipboard?id=clipboardsetobject"),
  "copy": $("fn(args: ?)", "Set clipboard, you can provide an expiration date.", "/foundation/clipboard?id=clipboardcopyobject"),
  "clear": $("fn()", "Clear all items in clipboard.", "/foundation/clipboard?id=clipboardclear"),
}, null, $(null, "Clipboard is very important for iOS data sharing, JSBox provides various interfaces.", "/foundation/clipboard"));

// $file
defs["$file"] = $class({
  "read": $("fn(path: string) -> $data", "Read a file.", "/file/method?id=filereadpath"),
  "write": $("fn(args: ?) -> bool", "Write a file.", "/file/method?id=filewriteobject"),
  "delete": $("fn(path: string) -> bool", "Delete a file.", "/file/method?id=filedeletepath"),
  "list": $("fn(folder: string) -> [string]", "Get all file names in a folder.", "/file/method?id=filelistpath"),
  "copy": $("fn(args: ?) -> bool", "Copy a file.", "/file/method?id=filecopyobject"),
  "move": $("fn(args: ?) -> bool", "Move a file.", "/file/method?id=filemoveobject"),
  "mkdir": $("fn(folder: string) -> bool", "Make a directory (folder).", "/file/method?id=filemkdirpath"),
  "exists": $("fn(path: string) -> bool", "Check if a file exists.", "/file/method?id=fileexistspath"),
  "isDirectory": $("fn(path: string) -> bool", "Check if a path is directory.", "/file/method?id=fileisdirectorypath"),
  "merge": $("fn(args: ?)", "Merge multiple files into one file.", "/file/method?id=filemerge"),
  "split": $("fn(args: ?) -> number", "Split a file into multiple files.", "/file/method?id=filesplit"),
}, null, $(null, "Provides some APIs to store/fetch files safely", "/file/method"));

// $drive
defs["$drive"] = $class({
  "open": $("fn() -> +Promise[:t=$data]", "Pick a file using iCloud Document Picker.", "/file/drive?id=driveopen"),
  "save": $("fn(args: ?) -> +Promise[:t=string]", "Save a file using Document Picker.", "/file/drive?id=drivesave"),
  "read": $("fn(path: string) -> $data", "Read a file.", "/file/method?id=filereadpath"),
  "write": $("fn(args: ?) -> bool", "Write a file.", "/file/method?id=filewriteobject"),
  "delete": $("fn(path: string) -> bool", "Delete a file.", "/file/method?id=filedeletepath"),
  "list": $("fn(folder: string) -> [string]", "Get all file names in a folder.", "/file/method?id=filelistpath"),
  "copy": $("fn(args: ?) -> bool", "Copy a file.", "/file/method?id=filecopyobject"),
  "move": $("fn(args: ?) -> bool", "Move a file.", "/file/method?id=filemoveobject"),
  "mkdir": $("fn(folder: string) -> bool", "Make a directory (folder).", "/file/method?id=filemkdirpath"),
  "exists": $("fn(path: string) -> bool", "Check if a file exists.", "/file/method?id=fileexistspath"),
  "isDirectory": $("fn(path: string) -> bool", "Checkc if a path is directory.", "/file/method?id=fileisdirectorypath"),
}, null, $(null, "Provides $drive in order to access iCloud Drive.", "/file/drive"));

// $cache
defs["$cache"] = $class({
  "set": $("fn(args: ?)", "Write to cache.", "/foundation/cache?id=cachesetstring-object"),
  "setAsync": $("fn(args: ?) -> +Promise[:t=?]", "Write to cache (async).", "/foundation/cache?id=cachesetasyncobject"),
  "get": $("fn(key: string) -> object", "Read from cache.", "/foundation/cache?id=cachegetstring"),
  "getAsync": $("fn(args: ?) -> +Promise[:t=?]", "Read from cache (async).", "/foundation/cache?id=cachegetasyncobject"),
  "remove": $("fn(key: string)", "Delete a cache.", "/foundation/cache?id=cacheremovestring"),
  "removeAsync": $("fn(args: ?) -> +Promise[:t=?]", "Delete a cache (async).", "/foundation/cache?id=cacheremoveasyncobject"),
  "clear": $("fn()", "Delete all cached objects.", "/foundation/cache?id=cacheclear"),
  "clearAsync": $("fn() -> +Promise[:t=?]", "Delete all cached objects (async).", "/foundation/cache?id=cacheclearasyncobject"),
}, null, $(null, "Cache is the easiest way to persistent data.", "/foundation/cache"));

// $context
defs["$context"] = $class({
  "query": $("object", "Get all query items (URL Scheme launched script).", "/context/method?id=contextquery"),
  "text": $("string", "Returns a text (user shared a text)", "/context/method?id=contexttext"),
  "textItems": $("[string]", "Returns all text items.", "/context/method?id=contexttextitems"),
  "link": $("string", "Returns a link (user shared a link).", "/context/method?id=contextlink"),
  "linkItems": $("[string]", "Returns all link items.", "/context/method?id=contextlinkitems"),
  "image": $("$image", "Returns an image (user shared an image).", "/context/method?id=contextimage"),
  "imageItems": $("[$image]", "Returns all image items.", "/context/method?id=contextimageitems"),
  "safari.items": $("object", "Returns all Safari items.", "/context/method?id=contextsafariitems"),
  "dataItems": $("[$data]", "Returns all binary files.", "/context/method?id=contextdataitems"),
  "allItems": $("object", "Returns all items, ignores the type.", "/context/method?id=contextallitems"),
  "close": $("fn()", "Close current action extension, the script stops running.", "/context/method?id=contextclose"),
}, null, $(null, "Fetch external data, that's very important to make an action extension.", "/context/method"));

// $timer
defs["$timer"] = $class({
  "schedule": $("fn(args: ?) -> $timer", "Schedule a timer.", "/foundation/thread?id=timerscheduleobject"),
}, null, $(null, "Schedule a timer.", "/foundation/thread?id=timerscheduleobject"));