var types = require("./types");
var defs = require("./defs");

// LayoutConstraint
types["$constraint"] = $type({
  "left": "$constraint",
  "top": "$constraint",
  "right": "$constraint",
  "bottom": "$constraint",
  "leading": "$constraint",
  "trailing": "$constraint",
  "width": "$constraint",
  "height": "$constraint",
  "centerX": "$constraint",
  "centerY": "$constraint",
  "baseline": "$constraint",
  "firstBaseline": "$constraint",
  "lastBaseline": "$constraint",
  "leftMargin": "$constraint",
  "rightMargin": "$constraint",
  "topMargin": "$constraint",
  "bottomMargin": "$constraint",
  "leadingMargin": "$constraint",
  "trailingMargin": "$constraint",
  "centerXWithinMargins": "$constraint",
  "centerYWithinMargins": "$constraint",
  "with": "$constraint",
  "and": "$constraint",
  "priorityToLow": "$constraint",
  "priorityToMedium": "$constraint",
  "priorityToHigh": "$constraint",
  "insets": "fn(insets: $insets) -> $constraint",
  "inset": "fn(inset: number) -> $constraint",
  "sizeOffset": "fn(size: $size) -> $constraint",
  "centerOffset": "fn(pt: $point) -> $constraint",
  "offset": "fn(offset: number) -> $constraint",
  "valueOffset": "fn(offset: $value) -> $constraint",
  "multipliedBy": "fn(ratio: number) -> $constraint",
  "dividedBy": "fn(ratio: number) -> $constraint",
  "priority": "fn(priority: number) -> $constraint",
  "equalTo": "fn(value: object) -> $constraint",
  "greaterThanOrEqualTo": "fn(value: object) -> $constraint",
  "lessThanOrEqualTo": "fn(value: object) -> $constraint",
}, null, $(null, "Layout constraint that is used to layout a view.", "/uikit/layout?id=what39s-constraint"));

// ConstraintMaker
types["$constraintMaker"] = $type({
  "left": "$constraint",
  "top": "$constraint",
  "right": "$constraint",
  "bottom": "$constraint",
  "leading": "$constraint",
  "trailing": "$constraint",
  "width": "$constraint",
  "height": "$constraint",
  "centerX": "$constraint",
  "centerY": "$constraint",
  "baseline": "$constraint",
  "firstBaseline": "$constraint",
  "lastBaseline": "$constraint",
  "leftMargin": "$constraint",
  "rightMargin": "$constraint",
  "topMargin": "$constraint",
  "bottomMargin": "$constraint",
  "leadingMargin": "$constraint",
  "trailingMargin": "$constraint",
  "centerXWithinMargins": "$constraint",
  "centerYWithinMargins": "$constraint",
  "edges": "$constraint",
  "size": "$constraint",
  "center": "$constraint",
  "updateExisting": "bool",
  "removeExisting": "bool",
}, null, $(null, "Constraint maker that can create layout constraints.", "/uikit/layout?id=properties"));

// Animator
types["$animator"] = $type({
  "transformIdentity": "$animator",
  "anchorDefault": "$animator",
  "anchorCenter": "$animator",
  "anchorTopLeft": "$animator",
  "anchorTopRight": "$animator",
  "anchorBottomLeft": "$animator",
  "anchorBottomRight": "$animator",
  "anchorTop": "$animator",
  "anchorBottom": "$animator",
  "anchorLeft": "$animator",
  "anchorRight": "$animator",
  "easeIn": "$animator",
  "easeOut": "$animator",
  "easeInOut": "$animator",
  "easeBack": "$animator",
  "spring": "$animator",
  "bounce": "$animator",
  "easeInQuad": "$animator",
  "easeOutQuad": "$animator",
  "easeInOutQuad": "$animator",
  "easeInCubic": "$animator",
  "easeOutCubic": "$animator",
  "easeInOutCubic": "$animator",
  "easeInQuart": "$animator",
  "easeOutQuart": "$animator",
  "easeInOutQuart": "$animator",
  "easeInQuint": "$animator",
  "easeOutQuint": "$animator",
  "easeInOutQuint": "$animator",
  "easeInSine": "$animator",
  "easeOutSine": "$animator",
  "easeInOutSine": "$animator",
  "easeInExpo": "$animator",
  "easeOutExpo": "$animator",
  "easeInOutExpo": "$animator",
  "easeInCirc": "$animator",
  "easeOutCirc": "$animator",
  "easeInOutCirc": "$animator",
  "easeInElastic": "$animator",
  "easeOutElastic": "$animator",
  "easeInOutElastic": "$animator",
  "easeInBack": "$animator",
  "easeOutBack": "$animator",
  "easeInOutBack": "$animator",
  "easeInBounce": "$animator",
  "easeOutBounce": "$animator",
  "easeInOutBounce": "$animator",
  "animating": "bool",
  "paused": "bool",
  "pause": "fn()",
  "resume": "fn()",
  "stop": "fn()",
  "makeFrame": "fn(rect: $rect) -> $animator",
  "makeBounds": "fn(rect: $rect) -> $animator",
  "makeSize": "fn(size: $size) -> $animator",
  "makeOrigin": "fn(pt: $point) -> $animator",
  "makeCenter": "fn(pt: $rect) -> $animator",
  "makeX": "fn(x: number) -> $animator",
  "makeY": "fn(y: number) -> $animator",
  "makeWidth": "fn(width: number) -> $animator",
  "makeHeight": "fn(height: number) -> $animator",
  "makeOpacity": "fn(opacity: number) -> $animator",
  "makeBackground": "fn(color: $color) -> $animator",
  "makeBorderColor": "fn(color: $color) -> $animator",
  "makeBorderWidth": "fn(width: number) -> $animator",
  "makeBackground": "fn(color: $color) -> $animator",
  "makeCornerRadius": "fn(radius: number) -> $animator",
  "makeScale": "fn(scale: number) -> $animator",
  "makeScaleX": "fn(scale: number) -> $animator",
  "makeScaleY": "fn(scale: number) -> $animator",
  "makeAnchor": "fn(pt: $point) -> $animator",
  "moveX": "fn(x: number) -> $animator",
  "moveY": "fn(y: number) -> $animator",
  "moveXY": "fn(pt: $point) -> $animator",
  "moveHeight": "fn(height: number) -> $animator",
  "moveWidth": "fn(width: number) -> $animator",
  "rotate": "fn(angle: number) -> $animator",
  "rotateX": "fn(angle: number) -> $animator",
  "rotateY": "fn(angle: number) -> $animator",
  "rotateZ": "fn(angle: number) -> $animator",
  "transformX": "fn(x: number) -> $animator",
  "transformY": "fn(y: number) -> $animator",
  "transformZ": "fn(z: number) -> $animator",
  "transformXY": "fn(pt: $point) -> $animator",
  "transformScale": "fn(scale: number) -> $animator",
  "transformScaleX": "fn(scale: number) -> $animator",
  "transformScaleY": "fn(scale: number) -> $animator",
  "delay": "fn(sec: number) -> $animator",
  "wait": "fn(sec: number) -> $animator",
  "preAnimationBlock": "fn(block: fn()) -> $animator",
  "animationBlock": "fn(block: fn()) -> $animator",
  "postAnimationBlock": "fn(block: fn()) -> $animator",
  "repeat": "fn(args: ?) -> $animator",
  "thenAfter": "fn(sec: number) -> $animator",
  "animate": "fn(sec: number) -> $animator",
  "animateWithRepeat": "fn(args: ?) -> $animator",
  "animateWithCompletion": "fn(args: ?) -> $animator",
}, null, $(null, "Animator provides chainable animations for a view.", "/uikit/animation?id=chainable"));