var types = require("./types");
var defs = require("./defs");

// $qrcode
defs["$qrcode"] = $class({
  "encode": $("fn(string: string) -> $image", "Encode a string to QRCode image.", "/extend/qrcode?id=qrcodeencodestring"),
  "decode": $("fn(image: $image) -> string", "Decode a string from QRCode image.", "/extend/qrcode?id=qrcodedecodeimage"),
  "scan": $("fn(args: ?) -> +Promise[:t=string]", "Scan QRCode with camera.", "/extend/qrcode?id=qrcodescanfunction"),
}, null, $(null, "QR Code related APIs.", "/extend/qrcode"));

// $detector
defs["$detector"] = $class({
  "date": $("fn(string: string) -> [+Date]", "Retrieve all dates from a string.", "/extend/detector?id=detectordatestring"),
  "address": $("fn(string: string) -> [string]", "Retrieve all addresses from a string.", "/extend/detector?id=detectoraddressstring"),
  "link": $("fn(string: string) -> [string]", "Retrieve all links from a string.", "/extend/detector?id=detectorlinkstring"),
  "phoneNumber": $("fn(string: string) -> [string]", "Retrieve all phone numbers from a string.", "/extend/detector?id=detectorphonenumberstring"),
}, null, $(null, "Enhances the readability of regex.", "/extend/detector"));

// $safari
defs["$safari"] = $class({
  "items": $("[?]", "Get items in Safari when you are using Action Extension.", "/sdk/safari?id=safariitems"),
  "open": $("fn(args: ?) -> +Promise[:t=?]", "Open website with Safari View Controller.", "/sdk/safari?id=safariopenobject"),
  "inject": $("fn(script: string)", "Inject JavaScript code to Safari when you are using Action Extension.", "/sdk/safari?id=safariinjectscript"),
  "addReadingItem": $("fn(args: ?) -> bool", "Add item to Safari reading list.", "/sdk/safari?id=safariaddreadingitemobject"),
}, null, $(null, "APIs related to Safari or Safari View Controller.", "/sdk/safari"));

// $archiver
defs["$archiver"] = $class({
  "zip": $("fn(args: ?) -> +Promise[:t=bool]", "Compress files to a zip.", "/extend/archiver?id=archiverzipobject"),
  "unzip": $("fn(args: ?) -> +Promise[:t=bool]", "Unzip a file.", "/extend/archiver?id=archiverunzipobject"),
}, null, $(null, "Built-in archiver/unarchiver module.", "/extend/archiver"));

// $keyboard
defs["$keyboard"] = $class({
  "hasText": $("bool", "Check whether current input context has text.", "/keyboard/method?id=keyboardhastext"),
  "selectedText": $("string", "Get current selected text (iOS 11 only).", "/keyboard/method?id=keyboardselectedtext"),
  "getAllText": $("fn(handler: fn(string: text)) -> +Promise[:t=string]", "Get all text (iOS 11 only).", "/keyboard/method?id=keyboardalltext"),
  "barHidden": $("bool", "Whether hides the bottom bar, this is a configuration, you should use it before the UI starts.", "/keyboard/method?id=keyboardbarhidden"),
  "insert": $("fn(string: string)", "Insert a string into current editing context.", "/keyboard/method?id=keyboardinsertstring"),
  "delete": $("fn()", "Delete selected text or delete backward.", "/keyboard/method?id=keyboarddelete"),
  "moveCursor": $("fn(offset: number)", "Move cursor by an offset.", "/keyboard/method?id=keyboardmovecursornumber"),
  "playInputClick": $("fn()", "Play sound effect of keyboard clicking.", "/keyboard/method?id=keyboardplayinputclick"),
  "next": $("fn()", "Switch to next keyboard.", "/keyboard/method?id=keyboardnext"),
  "send": $("fn()", "Simulate send action in the keyboard.", "/keyboard/method?id=keyboardsend"),
  "dismiss": $("fn()", "Dismiss the keyboard.", "/keyboard/method?id=keyboarddismiss"),
  "height": $("number", "Get or set keyboard height.", "/keyboard/method?id=keyboardheight"),
}, null, $(null, "Provides APIs to make a keyboard extension.", "/keyboard/method"));

// $text
defs["$text"] = $class({
  "tokenize": $("fn(args: ?) -> +Promise[:t=[string]]", "Text tokenization.", "/extend/text?id=texttokenizeobject"),
  "lookup": $("fn(text: string)", "Lookup text in system builtin dictionaries.", "/extend/text?id=textlookupstring"),
  "speech": $("fn(args: ?) -> $synthesizer", "Text to speech (TTS).", "/extend/text?id=textspeechobject"),
  "base64Encode": $("fn(text: string) -> string", "Base64 encode.", "/extend/text?id=textbase64encodestring"),
  "base64Decode": $("fn(text: string) -> string", "Base64 decode.", "/extend/text?id=textbase64decodestring"),
  "URLEncode": $("fn(text: string) -> string", "URL encode.", "/extend/text?id=texturlencodestring"),
  "URLDecode": $("fn(text: string) -> string", "URL decode.", "/extend/text?id=texturldecodestring"),
  "HTMLEscape": $("fn(text: string) -> string", "HTML escape.", "/extend/text?id=texthtmlescapestring"),
  "HTMLUnescape": $("fn(text: string) -> string", "HTML unescape.", "/extend/text?id=texthtmlunescapestring"),
  "MD5": $("fn(text: string) -> string", "MD5.", "/extend/text?id=textmd5string"),
  "SHA1": $("fn(text: string) -> string", "SHA1.", "/extend/text?id=textsha1string"),
  "SHA256": $("fn(text: string) -> string", "SHA256.", "/extend/text?id=textsha256string"),
  "convertToPinYin": $("fn(text: string) -> string", "Get Chinese PinYin of a string.", "/extend/text?id=textconverttopinyintext"),
  "markdownToHtml": $("fn(text: string) -> string", "Convert Markdown text to HTML text.", "/extend/text?id=textmarkdowntohtmltext"),
  "htmlToMarkdown": $("fn(args: ?) -> +Promise[:t=string]", "Convert HTML text to Markdown text.", "/extend/text?id=texthtmltomarkdownobject"),
  "decodeData": $("fn(args: ?) -> string", "Convert data to a string.", "/extend/text?id=textdecodedataobject"),
  "sizeThatFits": $("fn(args: ?) -> $size", "Calculate text bounding size dynamically.", "/extend/text?id=textsizethatfitsobject"),
}, null, $(null, "Provides a lot of utility functions to handle text.", "/extend/text"));

// $editor
defs["$editor"] = $class({
  "text": $("string", "All text in the editor.", "/extend/editor?id=editortext"),
  "selectedRange": $("$range", "Selected text range in the editor.", "/extend/editor?id=editorselectedrange"),
  "selectedText": $("string", "Selected text in the editor.", "/extend/editor?id=editorselectedtext"),
  "isActive": $("bool", "Whether the editor is active.", "/extend/editor?id=editorisactive"),
  "canUndo": $("bool", "Whether undo action can be taken.", "/extend/editor?id=editorcanundo"),
  "canRedo": $("bool", "Whether redo action can be taken.", "/extend/editor?id=editorcanredo"),
  "save": $("fn()", "Save changes in the code editor.", "/extend/editor?id=editorsave"),
  "undo": $("fn()", "Undo.", "/extend/editor?id=editorundo"),
  "redo": $("fn()", "Redo.", "/extend/editor?id=editorredo"),
  "activate": $("fn()", "Activate the current editor.", "/extend/editor?id=editoractivate"),
  "deactivate": $("fn()", "Deactivate the current editor.", "/extend/editor?id=editordeactivate"),
  "textInRange": $("fn(range: $range) -> string", "Get text in the given range.", "/extend/editor?id=editortextinrangerange"),
  "setTextInRange": $("fn(text: string, range: $range)", "Set text in the given range.", "/extend/editor?id=editorsettextinrangetextrange"),
}, null, $(null, "APIs that can be used to create plugins for the code editor.", "/extend/editor"));

// $xml

types["$xmlDoc"] = $type({
  "version": "string",
  "rootElement": "$xmlElement",
  "definePrefix": "fn(args: ?)",
  "enumerate": "fn(args: ?)",
  "firstChild": "fn(args: ?) -> $xmlElement",
  "children": "fn(args: ?) -> [$xmlElement]",
  "value": "fn(args: ?) -> object",
}, null, $(null, "XML/HTML document.", "/extend/xml?id=document"));

types["$xmlElement"] = $type({
  "document": "$xmlDoc",
  "blank": "bool",
  "namespace": "string",
  "tag": "string",
  "lineNumber": "number",
  "attributes": "object",
  "parent": "$xmlElement",
  "children": "[$xmlElement]",
  "previous": "$xmlElement",
  "next": "$xmlElement",
  "string": "string",
  "number": "number",
  "date": "+Date",
  "enumerate": "fn(args: ?)",
  "firstChild": "fn(args: ?) -> $xmlElement",
  "children": "fn(args: ?) -> [$xmlElement]",
  "value": "fn(args: ?) -> object",
}, null, $(null, "XML/HTML element.", "/extend/xml?id=element"));

defs["$xml"] = $class({
  "parse": $("fn(args: ?) -> $xmlDoc", "Parse a string as XML/HTML document, query with xPath or CSS selector.", "/extend/xml?id=xmlparseobject"),
}, null, $(null, "Provides XML/HTML parsing related functionality.", "/extend/xml"));