var types = require("./types");
var defs = require("./defs");

// SSH Session
types["$sshSession"] = $type({
  "hostConfig": "$sshHostConfig",
  "host": "string",
  "port": "number",
  "username": "string",
  "timeout": "number",
  "lastError": "$error",
  "fingerprintHash": "object",
  "banner": "string",
  "remoteBanner": "string",
  "connected": "bool",
  "authorized": "bool",
  "channel": "$sshChannel",
  "sftp": "$sftp",
}, null, $(null, "SSH session.", "/ssh/intro"));

// SSH Host Config
types["$sshHostConfig"] = $type({
  "hostPatterns": "[string]",
  "hostname": "string",
  "user": "string",
  "port": "number",
  "identityFiles": "[string]",
  "merge": $("fn(c: $sshHostConfig)", "Merge another config."),
}, null, $(null, "SSH host config.", "/ssh/intro"));

// SSH Channel
types["$sshChannel"] = $type({
  "session": "$sshSession",
  "bufferSize": "number",
  "type": "number",
  "lastResponse": "string",
  "requestPty": "bool",
  "ptyTerminalType": "number",
  "environmentVariables": "object",
  "execute": "fn(args: ?)" + $promise("$channelExecuteResult", {"response": "string", "error": "$error"}),
  "start": "fn(args: ?)" + $promise("$channelStartResult", {"success": "bool", "error": "$error"}),
  "close": "fn()",
  "write": "fn(args: ?)" + $promise("$channelWriteResult", {"success": "bool", "error": "$error"}),
  "requestSize": "fn(args: ?)" + $promise("$channelRequestSizeResult", {"width": "number", "height": "number"}),
  "upload": "fn(args: ?) -> +Promise[:t=bool]",
  "download": "fn(args: ?) -> +Promise[:t=bool]",
}, null, $(null, "SSH channel object.", "/ssh/channel"));

// SFTP
types["$sftp"] = $type({
  "session": "$sshSession",
  "connected": "bool",
  "bufferSize": "number",
  "connect": "fn(args: ?) -> +Promise[:t=bool]",
  "disconnect": "fn()",
  "moveItem": "fn(args: ?) -> +Promise[:t=bool]",
  "directoryExists": "fn(args: ?) -> +Promise[:t=bool]",
  "createDirectory": "fn(args: ?) -> +Promise[:t=bool]",
  "removeDirectory": "fn(args: ?) -> +Promise[:t=bool]",
  "contentsOfDirectory": "fn(args: ?) -> +Promise[:t=[$sftpFile]]",
  "infoForFile": "fn(args: ?) -> +Promise[:t=$sftpFile]",
  "fileExists": "fn(args: ?) -> +Promise[:t=bool]",
  "createSymbolicLink": "fn(args: ?) -> +Promise[:t=bool]",
  "removeFile": "fn(args: ?) -> +Promise[:t=bool]",
  "contents": "fn(args: ?) -> +Promise[:t=$data]",
  "write": "fn(args: ?) -> +Promise[:t=bool]",
  "append": "fn(args: ?) -> +Promise[:t=bool]",
  "copy": "fn(args: ?) -> +Promise[:t=bool]",
}, null, $(null, "SFTP object.", "/ssh/sftp"));

// SFTP File
types["$sftpFile"] = $type({
  "filename": "string",
  "isDirectory": "bool",
  "modificationDate": "+Date",
  "lastAccess": "+Date",
  "fileSize": "number",
  "ownerUserID": "number",
  "ownerGroupID": "number",
  "permissions": "string",
  "flags": "number",
}, null, $(null, "SFTP file object.", "/ssh/sftp"));

// WebSocket
$dict("$socketEvents", {
  "didOpen": "fn(sock: $socket)",
  "didFail": "fn(sock: $socket, err: $error)",
  "didClose": "fn(sock: $socket, code: number, reason: string, wasClean: bool)",
  "didReceiveMessage": "fn(sock: $socket, message: object)",
  "didReceiveString": "fn(sock: $socket, string: string)",
  "didReceiveData": "fn(sock: $socket, data: $data)",
  "didReceivePing": "fn(sock: $socket, data: $data)",
  "didReceivePong": "fn(sock: $socket, data: $data)",
  "shouldConvertToString": "fn(sock: $socket) -> bool",
});

$dict("$socketResult", { "result": "bool", "error": "$error" });

types["$socket"] = $type({
  "readyState": "number",
  "url": "$url",
  "protocol": "string",
  "allowsUntrustedSSLCertificates": "bool",
  "listen": $("fn(block: $socketEvents)", "Observe socket events."),
  "open": $("fn()", "Open the socket."),
  "close": $("fn(args: ?) -> bool", "Close the socket."),
  "send": $("fn(args: ?) -> $socketResult", "Send string or data."),
  "ping": $("fn(args: ?) -> $socketResult", "Send ping data."),
}, null, $(null, "Web Socket instance.", "/network/socket"));

// Web Server
$dict("$serverEvents", {
  "didStart": "fn(server: $server)",
  "didConnect": "fn(server: $server)",
  "didDisconnect": "fn(server: $server)",
  "didStop": "fn(server: $server)",
  "didCompleteBonjourRegistration": "fn(server: $server)",
  "didUpdateNATPortMapping": "fn(server: $server)",
});

$dict("$serverResult", { "result": "bool", "error": "$error" });

$dict("$serverHandlerRules", {
  "method": "string",
  "url": "string",
  "headers": "object",
  "path": "string",
  "query": "object",
});

$dict("$serverHandler", {
  "filter": "fn(rules: $serverHandlerRules) -> string",
  "response": "fn(request: $serverRequest) -> ?",
});

types["$server"] = $type({
  "running": "bool",
  "port": "number",
  "bonjourName": "string",
  "bonjourType": "string",
  "serverURL": "$url",
  "bonjourServerURL": "$url",
  "publicServerURL": "$url",
  "listen": $("fn(block: $serverEvents)", "Observe connection events."),
  "start": $("fn(args: ?) -> $serverResult", "Start the server."),
  "stop": $("fn()", "Stop the server."),
  "addHandler": $("fn(handler: $serverHandler)", "Add an event handler."),
  "clearHandlers": $("fn()", "Remove all handlers."),
}, null, $(null, "Web Server instance.", "/network/server"));

// Server Request
types["$serverRequest"] = $type({
  "method": "string",
  "url": "string",
  "headers": "object",
  "path": "string",
  "query": "object",
  "contentType": "string",
  "contentLength": "number",
  "ifModifiedSince": "+Date",
  "ifNoneMatch": "string",
  "byteRange": "$range",
  "acceptsGzip": "bool",
  "localAddressData": "$data",
  "localAddress": "string",
  "remoteAddressData": "$data",
  "remoteAddress": "string",
  "hasBody": "bool",
  "hasByteRange": "bool",
  "attributes": "object",
}, null, $(null, "Web server request.", "/network/server"));

// Server response
types["$serverResponse"] = $type({
  "contentType": "string",
  "contentLength": "number",
  "statusCode": "number",
  "cacheControlMaxAge": "number",
  "lastModifiedDate": "+Date",
  "eTag": "string",
  "gzipEnabled": "bool",
  "hasBody": "bool",
  "setHeader": $("fn(args: ?)", "Set http header field."),
}, null, $(null, "Web server response.", "/network/server"));

// $http

$dict("$httpRequest", {
  "method": "string",
  "url": "string",
  "header": "object",
  "body": "object",
  "handler": "fn(result: $httpResult)",
});

$dict("$httpResult", {
  "data": "string",
  "response": "$httpResponse",
  "error": "$error",
  "rawData": "$data",
});

$dict("$downloadResult", {
  "url": "string",
  "data": "$data",
  "status": "number",
  "response": "$httpResponse",
  "error": "$error",
});

defs["$http"] = $class({
  "status": $("number", "Get network status."),
  "request": $("fn(req: $httpRequest) -> +Promise[:t=$httpResult]", "Send a general HTTP request.", "/foundation/network?id=httprequestobject"),
  "get": $("fn(req: $httpRequest) -> +Promise[:t=$httpResult]", "Send a GET request.", "/foundation/network?id=httpgetobject"),
  "post": $("fn(req: $httpRequest) -> +Promise[:t=$httpResult]", "Send a POST request.", "/foundation/network?id=httppostobject"),
  "download": $("fn(req: $httpRequest) -> +Promise[:t=$downloadResult]", "Download a file with HTTP request.", "/foundation/network?id=httpdownloadobject"),
  "upload": $("fn(req: $httpRequest) -> +Promise[:t=$httpResult]", "Upload a file with HTTP request.", "/foundation/network?id=httpuploadobject"),
  "shorten": $("fn(args: ?) -> +Promise[:t=string]", "Shorten a link.", "/foundation/network?id=httpshortenobject"),
  "lengthen": $("fn(args: ?) -> +Promise[:t=string]", "Expand a link.", "/foundation/network?id=httplengthenobject"),
  "startServer": $("fn(args: ?)", "Start a web server, serve local files to provide HTTP interfaces.", "/foundation/network?id=httpstartserverobject"),
  "stopServer": $("fn()", "Stop a web server.", "/foundation/network?id=httpstopserver"),
}, null, $(null, "Network module is very important in JSBox, you can leverage this ability to achieve more.", "/foundation/network"));

// $network
defs["$network"] = $class({
  "ifa_data": $("object", "Get network interface received/sent data in bytes.", "/foundation/network?id=networkifa_data"),
  "proxy_settings": $("object", "Port of CFNetworkCopySystemProxySettings.", "/foundation/network?id=networkstartpingingobject"),
}, null, $(null, "Network module is very important in JSBox, you can leverage this ability to achieve more.", "/foundation/network"));

// $ssh
$dict("$sshConnectResult", {
  "session": "$sshSession",
  "response": "string",
});

defs["$ssh"] = $class({
  "connect": $("fn(args: ?) -> +Promise[:t=$sshConnectResult]", "Connect to server with password or keys, you can execute a script at the same time.", "/ssh/intro?id=sshconnectobject"),
  "disconnect": $("fn()", "Disconnect all SSH sessions connected by JSBox.", "/ssh/intro?id=sshdisconnect"),
}, null, $(null, "Secure Shell APIs.", "/ssh/intro"));

// $server
defs["$server"] = $class({
  "new": $("fn() -> $server", "Create a server with custom request handlers.", "/network/server?id=server"),
}, null, $(null, "Web server with custom request handlers.", "/network/server"));

// $socket
defs["$socket"] = $class({
  "new": $("fn() -> $socket", "Create a new socket connection.", "/network/socket?id=socketnewobject"),
}, null, $(null, "WebSocket like interfaces, it creates socket connection between client and server.", "/network/socket"));