var types = require("./types");
var defs = require("./defs");

// SpeechSynthesizer
types["$synthesizer"] = $type({
  "stop": "fn(boundary: number)",
  "pause": "fn(boundary: number)",
  "continue": "fn()",
}, null, $(null, "Speech synthesizer for dictation."));

// Contact Item
types["$contactItem"] = $type({
  "identifier": "string",
  "contactType": "number",
  "namePrefix": "string",
  "givenName": "string",
  "middleName": "string",
  "familyName": "string",
  "previousFamilyName": "string",
  "nameSuffix": "string",
  "nickname": "string",
  "organizationName": "string",
  "departmentName": "string",
  "jobTitle": "string",
  "phoneticGivenName": "string",
  "phoneticMiddleName": "string",
  "phoneticFamilyName": "string",
  "phoneticOrganizationName": "string",
  "note": "string",
  "imageData": "$data",
  "phoneNumbers": "[$labeledValue]",
  "emailAddresses": "[$labeledValue]",
  "postalAddresses": "[$labeledValue]",
  "urlAddresses": "[$labeledValue]",
  "contactRelations": "[$labeledValue]",
  "socialProfiles": "[$labeledValue]",
  "instantMessageAddresses": "[$labeledValue]",
  "birthday": "$dateComponents",
  "nonGregorianBirthday": "$dateComponents",
  "dates": "[$dateComponents]",
}, null, $(null, "Contact item in system Contacts App.", "/object/contact"));

// Labeled Value
types["$labeledValue"] = $type({
  "identifier": "string",
  "label": "string",
  "value": "object",
  "content": "string",
}, null, $(null, "Object that contains label and value."));

// Contact Group
types["$contactGroup"] = $type({
  "identifier": "string",
  "name": "string",
}, null, $(null, "Group in system Contacts App.", "/object/contact"));

// $push
defs["$push"] = $class({
  "schedule": $("fn(args: ?) -> +Promise[:t=?]", "Schedule a local notification.", "/extend/push?id=pushscheduleobject"),
  "cancel": $("fn(args: ?)", "Cancel a scheduled notification.", "/extend/push?id=pushcancelobject"),
}, null, $(null, "Used to schedule a notification or cancel a scheduled notification.", "/extend/push"));

// $input
defs["$input"] = $class({
  "text": $("fn(args: ?) -> +Promise[:t=string]", "Presents text field to receive inputs directly.", "/component/input?id=inputtextobject"),
  "speech": $("fn(args: ?) -> +Promise[:t=string]", "Speech to text.", "/component/input?id=inputspeechobject"),
}, null, $(null, "Receive user inputs easily.", "/component/input"));

// $picker
defs["$picker"] = $class({
  "date": $("fn(args: ?) -> +Promise[:t=+Date]", "Show date picker.", "/component/picker?id=pickerdateobject"),
  "data": $("fn(args: ?) -> +Promise[:t=?]", "Show general data picker.", "/component/picker?id=pickerdataobject"),
  "color": $("fn() -> +Promise[:t=$color]", "Show color picker."),
}, null, $(null, "Present picker views easily.", "/component/picker"));

// $motion
defs["$motion"] = $class({
  "available": $("bool", "Indicates whether the sensor is available."),
  "startUpdates": $("fn(args: ?)", "Start updates.", "/sdk/motion?id=motionstartupdatesobject"),
  "stopUpdates": $("fn()", "Stop updates.", "/sdk/motion?id=motionstopupdates"),
}, null, $(null, "Track sensor data updates.", "/sdk/motion"));

// $message
defs["$message"] = $class({
  "sms": $("fn(args: ?) -> +Promise[:t=number]", "Send text messages.", "/sdk/message?id=messagesmsobject"),
  "mail": $("fn(args: ?) -> +Promise[:t=number]", "Send email.", "/sdk/message?id=messagemailobject"),
}, null, $(null, "Send text messages and mails.", "/sdk/message"));

// $calendar
defs["$calendar"] = $class({
  "fetch": $("fn(args: ?) -> +Promise[:t=?]", "Fetch calendar items (request access first).", "/sdk/calendar?id=calendarfetchobject"),
  "create": $("fn(args: ?) -> +Promise[:t=?]", "Create a calendar item.", "/sdk/calendar?id=calendarcreateobject"),
  "save": $("fn(args: ?) -> +Promise[:t=?]", "Save modified calendar item.", "/sdk/calendar?id=calendarsaveobject"),
  "delete": $("fn(args: ?) -> +Promise[:t=?]", "Delete a calendar item.", "/sdk/calendar?id=calendardeleteobject"),
}, null, $(null, "Manage calendar items in Calendar App.", "/sdk/calendar"));

// $reminder
defs["$reminder"] = $class({
  "fetch": $("fn(args: ?) -> +Promise[:t=?]", "Fetch reminder items (request access first).", "/sdk/reminder?id=reminderfetchobject"),
  "create": $("fn(args: ?) -> +Promise[:t=?]", "Create a reminder item.", "/sdk/reminder?id=remindercreateobject"),
  "save": $("fn(args: ?) -> +Promise[:t=?]", "Save modified reminder item.", "/sdk/reminder?id=remindersaveobject"),
  "delete": $("fn(args: ?) -> +Promise[:t=?]", "Delete a reminder item.", "/sdk/reminder?id=reminderdeleteobject"),
}, null, $(null, "Manage reminder items in Reminders App.", "/sdk/reminder"));

// $contact
defs["$contact"] = $class({
  "pick": $("fn(args: ?) -> +Promise[:t=$contactItem|[$contactItem]]", "Pick one or multiple contact.", "/sdk/contact?id=contactpickobject"),
  "fetch": $("fn(args: ?) -> +Promise[:t=[$contactItem]]", "Fetch contacts with keywords.", "/sdk/contact?id=contactfetchobject"),
  "create": $("fn(args: ?) -> +Promise[:t=?]", "Create contact item.", "/sdk/contact?id=contactcreateobject"),
  "save": $("fn(args: ?) -> +Promise[:t=?]", "Save modified contact item.", "/sdk/contact?id=contactsaveobject"),
  "delete": $("fn(args: ?) -> +Promise[:t=?]", "Delete contacts.", "/sdk/contact?id=contactdeleteobject"),
  "fetchGroups": $("fn(args: ?) -> +Promise[:t=[$contactGroup]]", "Fetch all groups.", "/sdk/contact?id=contactfetchgroupsobject"),
  "addGroup": $("fn(args: ?) -> +Promise[:t=$contactGroup]", "Create a new group with name.", "/sdk/contact?id=contactaddgroupobject"),
  "deleteGroup": $("fn(args: ?) -> +Promise[:t=bool]", "Delete a group.", "/sdk/contact?id=contactdeletegroupobject"),
  "updateGroup": $("fn(args: ?) -> +Promise[:t=bool]", "Save updated group.", "/sdk/contact?id=contactupdategroupobject"),
  "addToGroup": $("fn(args: ?) -> +Promise[:t=bool]", "Add a contact to a group.", "/sdk/contact?id=contactaddtogroupobject"),
  "removeFromGroup": $("fn(args: ?) -> +Promise[:t=bool]", "Remove a contact from a group.", "/sdk/contact?id=contactremovefromgroupobject"),
}, null, $(null, "Manage contact items in Contacts App.", "/sdk/contact"));

// $location
defs["$location"] = $class({
  "available": $("bool", "Check whether location service is available.", "/sdk/location?id=locationavailable"),
  "fetch": $("fn(args: ?)", "Fetch location.", "/sdk/location?id=locationfetchobject"),
  "startUpdates": $("fn(args: ?)", "Track user location updates.", "/sdk/location?id=locationstartupdatesobject"),
  "trackHeading": $("fn(args: ?)", "Track heading data (compass).", "/sdk/location?id=locationtrackheadingobject"),
  "stopUpdates": $("fn()", "Stop updates.", "/sdk/location?id=locationstopupdates"),
  "select": $("fn() -> +Promise[:t=?]", "Select a location from iOS built-in Map.", "/sdk/location?id=locationselectobject"),
}, null, $(null, "Fetch/Track GPS information easily.", "/sdk/location"));

// $share
defs["$share"] = $class({
  "sheet": $("fn(args: ?)", "Using system builtin share sheet.", "/extend/share?id=sharesheetobject"),
  "wechat": $("fn(args: ?)", "Share content to WeChat.", "/extend/share?id=sharewechatobject"),
  "qq": $("fn(args: ?)", "Share content to QQ.", "/extend/share?id=shareqqobject"),
}, null, $(null, "Provides APIs to share media to social network services.", "/extend/share"));

// $intents
defs["$intents"] = $class({
  "finish": $("fn(result: string)", "Finish an asynchronous task in Siri/Shortcuts.", "/shortcuts/intents?id=run-jsbox-script"),
  "height": $("number", "Specify view height.", "/shortcuts/intents?id=specify-view-height"),
}, null, $(null, "Siri/Shortcuts APIs.", "/shortcuts/intro"));