var types = require("./types");
var defs = require("./defs");

// FMDB
types["$db"] = $type({
  "open": $("fn()", "Open database."),
  "close": $("fn()", "Close database."),
  "update": $("fn(args: ?) -> " + $dict("$dbUpdateResult", {"result": "bool", "error": "$error"}), "Execute database updates."),
  "query": $("fn(args: ?) -> " + $dict("$dbQueryResult", {"result": "$dbResultSet", "error": "$error"}), "Execute database query."),
  "beginTransaction": $("fn() -> bool", "Begin database transaction."),
  "rollback": $("fn() -> bool", "Rollback database transaction."),
  "commit": $("fn() -> bool", "Commit database transaction."),
}, null, $(null, "Database instance.", "/sqlite/usage"));

// FMDB Queue
types["$dbQueue"] = $type({
  "path": "string",
  "close": $("fn()", "Close database queue."),
  "interrupt": $("fn()", "Interrupt database queue."),
  "operations": $("fn(block: fn(db: $db))", "Perform batch updates on the queue."),
  "transaction": $("fn(block: fn(db: $db) -> bool)", "Perform transaction on the queue."),
}, null, $(null, "Database queue that provides support for multi-threads.", "/sqlite/queue"));

// FMDB ResultSet
types["$dbResultSet"] = $type({
  "query": "string",
  "columnCount": "number",
  "values": "object",
  "next": $("fn() -> bool", "Move the cursor to next."),
  "close": $("fn()", "Close the result set."),
  "indexForName": $("fn(name: string) -> number", "Get column index with name."),
  "nameForIndex": $("fn(index: number) -> string", "Get column name with index."),
  "get": $("fn(key: string|number) -> object", "Get column value of name or index."),
  "isNull": $("fn(key: string|number) -> bool", "Check if a column is null."),
}, null, $(null, "Result set that is returned from database query.", "/sqlite/usage"));

// $sqlite
defs["$sqlite"] = $class({
  "open": $("fn(path: string) -> $db", "Open a SQLite connection with file path.", "/sqlite/usage?id=sqliteopenpath"),
  "dbQueue": $("fn(path: string) -> $dbQueue", "Returns a database queue.", "/sqlite/queue"),
  "close": $("fn() -> bool", "Close a SQLite connection.", "/sqlite/usage?id=sqliteclosedb"),
}, null, $(null, "SQLite APIs.", "/sqlite/intro"));