var types = require("./types");
var defs = require("./defs");

// UIView
types["$view"] = $type({
  "alpha": "number",
  "bgcolor": "$color",
  "radius": "number",
  "smoothRadius": "number",
  "frame": "$rect",
  "bounds": "$rect",
  "userInteractionEnabled": "bool",
  "multipleTouchEnabled": "bool",
  "exclusiveTouch": "bool",
  "super": "$view",
  "prev": "$view",
  "next": "$view",
  "views": "[$view]",
  "window": "$view",
  "clipsToBounds": "bool",
  "opaque": "bool",
  "hidden": "bool",
  "contentMode": "number",
  "tintColor": "$color",
  "borderWidth": "number",
  "borderColor": "$color",
  "props": "object",
  "events": "object",
  "animator": "$animator",
  "snapshot": "$image",
  "id": "string",
  "left": "$layoutAttribute",
  "top": "$layoutAttribute",
  "right": "$layoutAttribute",
  "bottom": "$layoutAttribute",
  "leading": "$layoutAttribute",
  "trailing": "$layoutAttribute",
  "width": "$layoutAttribute",
  "height": "$layoutAttribute",
  "centerX": "$layoutAttribute",
  "centerY": "$layoutAttribute",
  "baseline": "$layoutAttribute",
  "firstBaseline": "$layoutAttribute",
  "lastBaseline": "$layoutAttribute",
  "leftMargin": "$layoutAttribute",
  "rightMargin": "$layoutAttribute",
  "topMargin": "$layoutAttribute",
  "bottomMargin": "$layoutAttribute",
  "leadingMargin": "$layoutAttribute",
  "trailingMargin": "$layoutAttribute",
  "centerXWithinMargins": "$layoutAttribute",
  "centerYWithinMargins": "$layoutAttribute",
  "safeArea": "$layoutAttribute",
  "safeAreaLeading": "$layoutAttribute",
  "safeAreaTrailing": "$layoutAttribute",
  "safeAreaLeft": "$layoutAttribute",
  "safeAreaRight": "$layoutAttribute",
  "safeAreaTop": "$layoutAttribute",
  "safeAreaBottom": "$layoutAttribute",
  "safeAreaWidth": "$layoutAttribute",
  "safeAreaHeight": "$layoutAttribute",
  "safeAreaCenterX": "$layoutAttribute",
  "safeAreaCenterY": "$layoutAttribute",
  "layout": $("fn(block: fn(make: $constraintMaker, view: $view)) -> [$constraint]", "Add layout constraints to a view."),
  "updateLayout": $("fn(block: fn(make: $constraintMaker, view: $view)) -> [$constraint]", "Update layout constraints of a view."),
  "remakeLayout": $("fn(block: fn(make: $constraintMaker, view: $view)) -> [$constraint]", "Recreate layout constraints to a view."),
  "add": $("fn(def: object)", "Add a subview."),
  "get": $("fn(id: string) -> $view", "Get subview with identifier."),
  "remove": $("fn()", "Remove from super view's hierarchy."),
  "relayout": $("fn()", "Trigger relayout, also applies to all subviews."),
  "scale": $("fn(ratio: number)", "Scale the view."),
  "rotate": $("fn(angle: number)", "Rotate the view."),
  "bounce": $("fn()", "Creates a bounce effect of the view."),
  "startLoading": $("fn()", "Start loading."),
  "stopLoading": $("fn()", "Stop loading."),
  "info": $("fn() -> object", "Get associated objects in the view."),
}, null, $(null, "Base view in UIKit, it's the base class of all views.", "/uikit/view"));

// UIControl
types["$controlView"] = $type({
  "enabled": "bool",
  "selected": "bool",
  "highlighted": "bool",
  "contentVerticalAlignment": "number",
  "contentHorizontalAlignment": "number",
  "effectiveContentHorizontalAlignment": "number",
  "state": "number",
  "tracking": "bool",
  "touchInside": "bool",
}, types["$view"], $(null, "A subclass of $view, provides user interactions."));

// UILabel
types["$labelView"] = $type({
  "text": "string",
  "font": "$font",
  "textColor": "$color",
  "shadowColor": "$color",
  "shadowOffset": "$size",
  "align": "$align",
  "lines": "number",
  "autoFontSize": "bool",
}, types["$view"], $(null, "Label is used to display simple text, not editable.", "/component/label"));

// UIButton
types["$button"] = $type({
  "contentEdgeInsets": "$insets",
  "titleEdgeInsets": "$insets",
  "imageEdgeInsets": "$insets",
  "type": "number",
  "title": "string",
  "titleColor": "$color",
  "font": "$font",
  "src": "string",
  "image": "$image",
  "imageData": "$data",
  "icon": "$icon",
}, types["$controlView"], $(null, "Button is used to handle tap events.", "/component/button"));

// BBTabView
types["$tabView"] = $type({
  "items": "[string]",
  "index": "number",
  "enable": $("fn(index: number)", "Enable item at index."),
  "disable": $("fn(index: number)", "Disable item at index."),
}, types["$controlView"], $(null, "Tab that contains several menu items.", "/component/menu"));

// UITextField
types["$inputView"] = $type({
  "type": "$kbType",
  "returnKeyType": "number",
  "darkKeyboard": "bool",
  "text": "string",
  "textColor": "$color",
  "font": "$font",
  "align": "$align",
  "placeholder": "string",
  "clearsOnBeginEditing": "bool",
  "autoFontSize": "bool",
  "editing": "bool",
  "clearButtonMode": "number",
  "secure": "bool",
  "keyboardView": "$view",
  "accessoryView": "$view",
  "focus": $("fn()", "Focus, show the keyboard."),
  "blur": $("fn()", "Unfocus, dismiss the keyboard."),
}, types["$controlView"], $(null, "Single line input component.", "/component/input"));

// UISlider
types["$sliderView"] = $type({
  "value": "number",
  "min": "number",
  "max": "number",
  "continuous": "bool",
  "minColor": "$color",
  "maxColor": "$color",
  "thumbColor": "$color",
  "thumbImage": "$image",
}, types["$controlView"], $(null, "Provides a slider control, values can be changed continuously.", "/component/slider"));

// UISwitch
types["$switchView"] = $type({
  "onColor": "$color",
  "thumbColor": "$color",
  "on": "bool",
}, types["$controlView"], $(null, "Provides two states: on/off, like a switch in real life.", "/component/switch"));

// Indicator
types["$spinnerView"] = $type({
  "loading": "bool",
  "start": $("fn()", "Start loading."),
  "stop": $("fn()", "Stop loading."),
  "toggle": $("fn()", "Toggle loading state."),
}, types["$view"], $(null, "Indicator view that shows loading state.", "/component/spinner"));

// UIProgressView
types["$progressView"] = $type({
  "value": "number",
  "progressColor": "$color",
  "trackColor": "$color",
}, types["$view"], $(null, "Progress view that displays progress.", "/component/progress"));

// UIStepper
types["$stepperView"] = $type({
  "continuous": "bool",
  "autorepeat": "bool",
  "wraps": "bool",
  "value": "number",
  "min": "number",
  "max": "number",
  "step": "number",
}, types["$controlView"], $(null, "Stepper view that provides +/- buttons.", "/component/stepper"));

// UIImageView
types["$imageView"] = $type({
  "image": "$image",
  "data": "$data",
  "path": "string",
  "src": "string",
  "icon": "$icon",
  "highlighted": "bool",
  "set": $("fn(args: ?)", "Setup image view.")
}, types["$view"], $(null, "Image view that displays image.", "/component/image"));

// UIScrollView
types["$scrollView"] = $type({
  "contentOffset": "$point",
  "contentSize": "$size",
  "contentInset": "$insets",
  "directionalLockEnabled": "bool",
  "alwaysBounceVertical": "bool",
  "alwaysBounceHorizontal": "bool",
  "pagingEnabled": "bool",
  "scrollEnabled": "bool",
  "showsHorizontalIndicator": "bool",
  "showsVerticalIndicator": "bool",
  "indicatorInsets": "$insets",
  "indicatorStyle": "number",
  "tracking": "bool",
  "dragging": "bool",
  "decelerating": "bool",
  "delaysContentTouches": "bool",
  "keyboardDismissMode": "number",
  "beginRefreshing": $("fn()", "Begin refreshing, shows loading spinner."),
  "endRefreshing": $("fn()", "End refreshing, hides loading spinner."),
  "resize": $("fn()", "Resize frame to fit the content."),
  "scrollToOffset": $("fn(offset: $point)", "Scroll to offset."),
}, types["$view"], $(null, "Scrollable view, shows long content.", "/component/scroll"));

// UITextView
types["$textView"] = $type({
  "type": "$kbType",
  "returnKeyType": "number",
  "darkKeyboard": "bool",
  "text": "string",
  "html": "string",
  "font": "$font",
  "textColor": "$color",
  "align": "$align",
  "selectedRange": "$range",
  "editable": "bool",
  "selectable": "bool",
  "insets": "$insets",
  "keyboardView": "$view",
  "accessoryView": "$view",
  "focus": $("fn()", "Focus, show the keyboard."),
  "blur": $("fn()", "Unfocus, dismiss the keyboard."),
}, types["$scrollView"], $(null, "TextView provides editable text area.", "/component/text"));

// MapView
types["$mapView"] = $type({
  "location": "object",
}, types["$view"], $(null, "MapView that displays built-in map.", "/component/map"));

// WKWebView
types["$webView"] = $type({
  "url": "string",
  "html": "string",
  "text": "string",
  "loading": "bool",
  "progress": "number",
  "canGoBack": "bool",
  "canGoForward": "bool",
  "ua": "string",
  "toolbar": "bool",
  "scrollView": "$scrollView",
  "scrollEnabled": "bool",
  "transparent": "bool",
  "bounces": "bool",
  "goBack": $("fn()", "Go back."),
  "goForward": $("fn()", "Go forward."),
  "reload": $("fn()", "Reload the web page."),
  "reloadFromOrigin": $("fn()", "Reload the web page from origin."),
  "stopLoading": $("fn()", "Stop loading the web page."),
  "eval": $("fn(args: ?) -> +Promise[:t=?]", "Evaluate script on webView."),
  "notify": $("fn(args: ?) -> +Promise[:t=?]", "Notify native environment."),
}, types["$view"], $(null, "WebView that is used to display web contents.", "/component/web"));

// DatePickerView
types["$datePicker"] = $type({
  "date": "+Date",
  "mode": "number",
  "interval": "number",
  "min": "+Date",
  "max": "+Date",
}, types["$view"], $(null, "Provides a picker view to select date easily.", "/component/date-picker"));

// PickerView
types["$pickerView"] = $type({
  "data": "[?]",
  "selectedRows": "[?]",
}, types["$view"], $(null, "Provides a generic picker view to select data.", "/component/picker"));

// HMSegmentedControl
types["$menuView"] = $type({
  "items": "[string]",
  "index": "number",
}, types["$controlView"], $(null, "Menu that contains many menu items, the difference between `tab` and `menu` is that `menu` is scrollable.", "/component/menu"));

// MarkdownViewer
types["$markdownView"] = $type({
  "content": "string",
  "scrollEnabled": "bool",
  "webView": "$webView",
}, types["$view"], $(null, "Markdown view is designed to display markdown format content.", "/component/markdown"));

// RefreshControl
types["$refreshControl"] = $type({
  "refreshing": "bool",
  "tintColor": "$color",
  "beginRefreshing": $("fn()", "Begin refreshing."),
  "endRefreshing": $("fn()", "End refreshing."),
}, types["$controlView"], $(null, "Refresh control indicates that refreshing state."));

// GalleryView
types["$galleryView"] = $type({
  "page": "number",
  "interval": "number",
  "items": "[?]",
}, types["$scrollView"], $(null, "Provides a scrollable page control."));

// GradientView
types["$gradientView"] = $type({
  "colors": "[$color]",
  "locations": "[$point]",
  "startPoint": "$point",
  "endPoint": "$point",
}, types["$view"], $(null, "Provides gradient layer."));

// MatrixView
types["$matrixView"] = $type({
  "data": "[?]",
  "selectable": "bool",
  "reorder": "bool",
  "object": "fn(indexPath: $indexPath) -> object",
  "insert": "fn(args: ?)",
  "delete": "fn(index: number)",
  "cell": "fn(indexPath: ?) -> $view",
  "scrollTo": "fn(args: ?)",
  "reload": "fn()",
}, types["$scrollView"], $(null, "Provides grid-like layout system."));

// TableView
types["$listView"] = $type({
  "separatorInset": "$insets",
  "data": "[string]",
  "header": "$view",
  "footer": "$view",
  "separatorHidden": "bool",
  "separatorColor": "$color",
  "reorder": "bool",
  "crossSections": "bool",
  "selectable": "bool",
  "object": "fn(indexPath: $indexPath) -> object",
  "insert": "fn(args: ?)",
  "delete": "fn(index: number)",
  "cell": "fn(indexPath: ?) -> $view",
  "scrollTo": "fn(args: ?)",
  "setEditing": "fn(editing: bool)",
  "reload": "fn()",
}, types["$scrollView"], $(null, "Provides list view to reuse cells."));

// VideoView
types["$videoView"] = $type({
  "poster": "string",
  "src": "string",
  "pause": "fn()",
  "play": "fn()",
  "toggle": "fn()",
}, types["$webView"], $(null, "A simple wrapper of WebView, provides video player."));

// LottieView
types["$lottieView"] = $type({
  "json": "object",
  "data": "$data",
  "src": "string",
  "playing": "bool",
  "loop": "bool",
  "autoReverse": "bool",
  "progress": "number",
  "frame": "number",
  "speed": "number",
  "duration": "number",
  "cacheEnable": "bool",
  "model": "object",
  "shouldRasterize": "bool",
  "play": "fn(args: ?)",
  "pause": "fn()",
  "stop": "fn()",
  "update": "fn()",
}, types["$view"], $(null, "Lottie view, provides animations."));

// LottieModel
types["$lottieModel"] = $type({
  "compBounds": "$rect",
  "startFrame": "number",
  "endFrame": "number",
  "framerate": "number",
  "timeDuration": "number",
}, null, $(null, "Lottie animation definition."));

// ChartView
types["$chartView"] = $type({
  "options": "object",
  "render": "fn(args: ?)",
  "eval": "fn(args: ?) -> +Promise[:t=?]",
  "notify": "fn(args: ?) -> +Promise[:t=?]",
  "dispatchAction": "fn(args: ?)",
  "getWidth": "fn() -> +Promise[:t=number]",
  "getHeight": "fn() -> +Promise[:t=number]",
  "getOption": "fn() -> +Promise[:t=?]",
  "resize": "fn(size: $size)",
  "showLoading": "fn()",
  "hideLoading": "fn()",
  "clear": "fn()",
}, types["$view"], $(null, "Chart view for data visualization."));

// UIKit

$dict("$selectResult", {
  "title": "string",
  "index": "number",
});

defs["$ui"] = $class({
  "window": $("$view", "Get current window of $ui.render.", "/uikit/method?id=uiwindow"),
  "controller": $("object", "Get the most front view controller of the app.", "/uikit/method?id=uicontroller"),
  "title": $("string", "Get or set the most front view's title.", "/uikit/method?id=uititle"),
  "create": $("fn(args: ?) -> $view", "Create a view manually.", "/uikit/method?id=uicreateobject"),
  "render": $("fn(args: ?) -> [$view]", "Render views on the screen.", "/uikit/render?id=uirenderobject"),
  "push": $("fn(args: ?) -> [$view]", "Push a new root view above previous view.", "/uikit/render?id=uipushobject"),
  "pop": $("fn()", "Pop out the most front view from the stack.", "/uikit/method?id=uipop"),
  "popToRoot": $("fn()", "Pop to the root view.", "/uikit/method?id=uipoptoroot"),
  "get": $("fn(id: string) -> $view", "Get a view instance by id, same as $(id).", "/uikit/method?id=uigetid"),
  "animate": $("fn(args: ?) -> +Promise[:t=bool]", "Create simple animation on views.", "/uikit/animation?id=uiview-animation"),
  "alert": $("fn(args: ?) -> +Promise[:t=$selectResult]", "Present an alert, usually used to show a message or options.", "/uikit/method?id=uialertobject"),
  "action": $("fn(args: ?) -> +Promise[:t=$selectResult]", "Present an action sheet, all parameters are same as $ui.alert.", "/uikit/method?id=uiactionobject"),
  "menu": $("fn(items: [string]) -> +Promise[:t=$selectResult]", "Present a menu, it can be used to provide a series of options.", "/uikit/method?id=uimenuobject"),
  "toast": $("fn(message: string, delay?: number)", "Show a toast message at top of the root view.", "/uikit/method?id=uitoastmessage"),
  "clearToast": $("fn()", "You can clear toast with this.", "/uikit/method?id=uitoastmessage"),
  "error": $("fn(message: string, delay?: number)", "Show an error message.", "/uikit/method?id=uitoastmessage"),
  "loading": $("fn(args: bool|string)", "Show a loading indicator.", "/uikit/method?id=uiloadingboolean"),
  "progress": $("fn(value: number, message?: string)", "Display a progress bar, the range of value is [0, 1].", "/uikit/method?id=uiprogressnumber"),
  "preview": $("fn(args: ?)", "Preview a url quickly.", "/uikit/method?id=uipreviewobject"),
  "selectIcon": $("fn() -> +Promise[:t=string]", "Select built-in icon.", "/uikit/method?id=uiselecticon"),
}, null, $(null, "Build user interface with simple APIs.", "/uikit/intro"));